package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.JobService;
import lk.sliit.research.interviewapp.model.domain.Job;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import static lk.sliit.research.interviewapp.util.Constant.APP_NAME;
import static lk.sliit.research.interviewapp.util.Constant.ERROR_LOG;

@Slf4j
@RestController
@RequestMapping("job")
public class JobController {
    @Autowired
    JobService jobService;

    @GetMapping("addJobView")
    public ModelAndView addJobView(){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("job/addJobView");
            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @GetMapping("isUniqueName/{name}")
    public boolean isUniqueName(@PathVariable String name){
        log.info("Request: {}", name);

        try{
            return jobService.isUniqueName(name);
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return false;
        }
    }

    @PostMapping("createJob")
    public boolean createJob(@RequestBody Job job){
        log.info("Request: {}", job);

        try{
             return jobService.createJob(job);
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return false;
        }
    }

    @GetMapping("listAllJobsView")
    public ModelAndView listAllJobsView(){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.addObject("jobList", jobService.listAllJobs());
            modelAndView.setViewName("job/listAllJobs");
            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("listAllJobsFilter")
    public ModelAndView listAllJobsFilter(@RequestParam("name") String name){
        log.info("name: {}", name);

        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.addObject("jobList", jobService.filterJobsOnName(name));
            modelAndView.setViewName("job/listAllJobs");
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }
}
