package lk.sliit.research.interviewapp.Jobs;

import lk.sliit.research.interviewapp.Service.CandidateService;
import lk.sliit.research.interviewapp.Service.InterviewService;
import lk.sliit.research.interviewapp.Service.QnaInterviewService;
import lk.sliit.research.interviewapp.model.domain.Candidate;
import lk.sliit.research.interviewapp.model.domain.Interview;
import lk.sliit.research.interviewapp.model.domain.QnaInterview;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.List;

import static lk.sliit.research.interviewapp.util.Constant.*;

@Slf4j
@Service
public class JobService {

    @Autowired
    Environment env;

    @Autowired
    CandidateService candidateService;

    @Autowired
    InterviewService interviewService;

    @Autowired
    QnaInterviewService qnaInterviewService;

    @Scheduled(fixedRateString  = "30000")
    public void resumeAnalyzer(){
        try{
            String jobStatus = env.getProperty("job.resume.analyzer.status");
            if(jobStatus.equalsIgnoreCase(ON)){
                /**
                 * Fetch x resumes at a time
                 * */
                int fetchSize = Integer.parseInt(env.getProperty("job.resume.analyzer.resume.fetch"));
                log.info("Fetch size of resumeAnalyzer: {}", fetchSize);
                List<Candidate> candidateList = candidateService.viewCandidateForResumeAnalysis(fetchSize);

                if(candidateList.size() == 0){
                    log.info("There is no candidate's resume to analyze");
                }
                else{
                    for(Candidate candidate : candidateList){
                        try{
                            log.info("resumeAnalyzer processing starter!");
                            log.info("Candidate: {}", candidate);
                            candidate.setResumeAnalysis(CANDIDATE_STATUS_PROCESSING);
                            candidateService.updateCandidate(candidate);

                            /**
                             * Resume analysis - external API - mock
                             * */
                            Thread.sleep(5000);
                            double score = 85;

                            candidate.setResumeAnalysis(CANDIDATE_STATUS_DONE);
                            candidate.setResumeAnalysisResponse("Resume analysis done!");
                            candidate.setScore(score);
                            candidateService.updateCandidate(candidate);
                        }
                        catch(Exception e){
                            candidate.setResumeAnalysis(CANDIDATE_STATUS_ERROR_ANALYSIS);
                            candidate.setResumeAnalysisResponse("Failed job analysis: " + e.getMessage());
                            candidateService.updateCandidate(candidate);
                            log.error(ERROR_LOG, e);
                        }
                    }
                }
            }
            else{
                log.info("Job(resumeAnalyzer) is in off state...");
            }
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
        }

    }

    @Scheduled(fixedRateString  = "30000")
    public void interviewAnalyzer(){
        try{
            String jobStatus = env.getProperty("job.interview.analyzer.status");
            if(jobStatus.equalsIgnoreCase(ON)){
                /**
                 * Fetch x interviews at a time
                 * */
                int fetchSize = Integer.parseInt(env.getProperty("job.interview.analyzer.fetch"));
                log.info("Fetch size of interviewAnalyzer: {}", fetchSize);
                List<Interview> interviewList = interviewService.viewInterviewForAnalysis(fetchSize);

                if(interviewList.size() == 0){
                    log.info("There is no interviews to analyze");
                }
                else{
                    for(Interview interview : interviewList){
                        try{
                            log.info("interviewAnalyzer processing starter!");
                            log.info("Interview: {}", interview);
                            interviewService.updateInterviewLifeCycle(interview.getId(), INTERVIEW_STATUS_SUBMITTED_FOR_RESULT);

                            List<QnaInterview> qnaInterviewList = qnaInterviewService.findByInterviewId(interview.getId());
                            for(QnaInterview qnaInterview : qnaInterviewList){
                                try{
                                    /**
                                     * QNA analysis - external API - mock
                                     * */
                                    qnaInterview.setScore(85);
                                    qnaInterview.setStatus(QNA_STATUS_CALCULATED);
                                }
                                catch (Exception e){
                                    log.error(ERROR_LOG, e);
                                    qnaInterview.setScore(0);
                                    qnaInterview.setStatus(QNA_STATUS_ERROR_ANALYSIS);
                                }
                            }

                            /**
                             * Interview analysis - external API - mock
                             * */
                            Thread.sleep(5000);
                            double interviewScore = 85;

                            /**
                             * Batch update QNA
                             * */
                            qnaInterviewService.saveOrUpdateBatchQnaInterview(qnaInterviewList);

                            /**
                             * Update interview details
                             * */
                            interview.setStatus(INTERVIEW_STATUS_TO_BE_SELECTED);
                            interview.setInterviewAnalysisResponse("Interview analysis done!");
                            interview.setScore(interviewScore);
                            interview.setComment("A good candidate!");
                            interviewService.saveOrUpdate(interview);
                        }
                        catch(Exception e){
                            interview.setStatus(INTERVIEW_STATUS_ERROR_ANALYSIS);
                            interview.setInterviewAnalysisResponse("Failed job analysis: " + e.getMessage());
                            interviewService.saveOrUpdate(interview);
                            log.error(ERROR_LOG, e);
                        }
                    }
                }
            }
            else{
                log.info("Job(interviewAnalyzer) is in off state...");
            }
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
        }
    }

}
