package lk.sliit.research.interviewapp.Service.impl;

import lk.sliit.research.interviewapp.Repository.CandidateRepository;
import lk.sliit.research.interviewapp.Repository.InterviewRepository;
import lk.sliit.research.interviewapp.Repository.JobRepository;
import lk.sliit.research.interviewapp.Service.InterviewService;
import lk.sliit.research.interviewapp.model.domain.Candidate;
import lk.sliit.research.interviewapp.model.domain.Interview;
import lk.sliit.research.interviewapp.model.domain.Job;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import static lk.sliit.research.interviewapp.util.Constant.*;

@Slf4j
@Service
public class InterviewServiceImpl implements InterviewService {

    @Autowired
    private InterviewRepository interviewRepository;

    @Autowired
    private JobRepository jobRepository;

    @Autowired
    private CandidateRepository candidateRepository;

    @Override
    public boolean isValidInterviewKey(String key) {
        return (interviewRepository.countInterviewByGivenKey(key) > 0);
    }

    @Override
    public boolean updateInterviewLifeCycle(int interviewId, String status) {
        Interview interview = interviewRepository.findById(interviewId).get();
        interview.setStatus(status);

        if(status.equalsIgnoreCase(INTERVIEW_STATUS_PROCESSING)){
            Date date = new Date();
            SimpleDateFormat formatter = new SimpleDateFormat(DATE_FORMAT);
            String dateStr = formatter.format(date);
            interview.setJoinDate(dateStr);
        }

        interviewRepository.save(interview);

        return true;
    }

    @Override
    public Interview getInterviewByRoomIdAndKey(String roomId, String key) {
        return interviewRepository.findByRoomIdAndKey(roomId, key);
    }

    @Override
    public Interview getInterviewByCandidateId(int candidateId) {
        return interviewRepository.findByCandidateId(candidateId);
    }

    @Override
    public boolean placeInterview(Interview interview) {
        interview.setStatus(INTERVIEW_STATUS_INITIAL);
        interview.setScore(0);
        interviewRepository.save(interview);
        return true;
    }

    @Override
    public String checkInterviewEligible(int candidateId) {
        StringBuilder message = new StringBuilder();

        boolean isJobAdvertisementExist = true;
        boolean isResumeAnalyzed = true;
        boolean isDuplicatedInterview = true;

        /** Resume analyse logic */
        Candidate candidate = candidateRepository.findById(candidateId).get();
        if(!candidate.getResumeAnalysis().equalsIgnoreCase(CANDIDATE_STATUS_DONE))
            isResumeAnalyzed = false;

        /** Job advertisement exist logic */
        Job job = jobRepository.findById(candidate.getJobId()).get();
        if(job.getFile() == null)
            isJobAdvertisementExist = false;

        /** Duplicated interview */
        Interview interview = interviewRepository.findByCandidateId(candidateId);
        if(interview != null)
            isDuplicatedInterview = false;

        if(!isJobAdvertisementExist)
            message.append("(Job advertisement doesn't exist!)");

        if(!isResumeAnalyzed)
            message.append("(Resume is not analysed yet!)");

        if(!isDuplicatedInterview)
            message.append("(This candidate has an interview scheduled!)");

        if(isJobAdvertisementExist && isResumeAnalyzed && isDuplicatedInterview)
            message.append(SUCCESS);

        return message.toString();
    }

    @Override
    public List<Interview> viewInterviewForAnalysis(int limit) {
        return interviewRepository.viewInterviewForAnalysis(limit);
    }

    @Override
    public boolean saveOrUpdate(Interview interview) {
        interviewRepository.save(interview);
        return true;
    }

    @Override
    public Interview findByRoomIdAndKeyNotInInitial(String roomId, String key) {
        return interviewRepository.findByRoomIdAndKeyNotInInitial(roomId, key);
    }

    @Override
    public boolean updateInterviewSelectionStatus(int candidateId, String status) {
        if(status.equalsIgnoreCase(INTERVIEW_STATUS_SELECTED)
                || status.equalsIgnoreCase(INTERVIEW_STATUS_REJECTED)){
            Interview interview = getInterviewByCandidateId(candidateId);
            interview.setStatus(status);

            saveOrUpdate(interview);

            return true;
        }
        else{
            log.info("Failed to update interview status. Status should be: {} or {}", INTERVIEW_STATUS_SELECTED, INTERVIEW_STATUS_REJECTED);
            return false;
        }
    }
}
