package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.CandidateService;
import lk.sliit.research.interviewapp.Service.InterviewService;
import lk.sliit.research.interviewapp.model.domain.Candidate;
import lk.sliit.research.interviewapp.model.domain.Interview;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Random;

import static lk.sliit.research.interviewapp.util.Constant.*;

@Slf4j
@RestController
@RequestMapping("candidate")
public class CandidateController {

    private static String UPLOADED_FOLDER = "Interview-App-Git//interview-app//src//main//webapp//view//uploads//resume//";
    private static String UPLOADED_FOLDER_CANDIDATE_IMAGE = "Interview-App-Git//interview-app//src//main//webapp//view//uploads//candidate//image//";

    @Autowired
    CandidateService candidateService;

    @Autowired
    InterviewService interviewService;

    @Autowired
    Environment env;

    @GetMapping("addCandidateView/{jobId}")
    public ModelAndView addCandidateView(@PathVariable int jobId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("candidate/addCandidateView");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("jobId", jobId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("createCandidate")
    public boolean createCandidate(@RequestBody Candidate candidate){
        log.info("Request: {}", candidate);

        try{
            return candidateService.createCandidate(candidate);
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return false;
        }
    }

    @GetMapping("viewAllCandidates/{jobId}")
    public ModelAndView viewAllCandidates(@PathVariable int jobId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.addObject("candidateList", candidateService.viewAllCandidatesByJobId(jobId));
            modelAndView.setViewName("candidate/listAllCandidates");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("jobId", jobId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @GetMapping("viewUpdateResumeView/{candidateId}")
    public ModelAndView viewUpdateResumeView(@PathVariable int candidateId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            Candidate candidate = candidateService.getCandidateById(candidateId);
            Interview interview = interviewService.getInterviewByCandidateId(candidateId);

            modelAndView.setViewName("candidate/viewUpdateCandidateView");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("resumeAnalysis", candidate.getResumeAnalysis());
            if(interview == null){
                modelAndView.addObject("interviewAnalysis", "NONE");
            }
            else{
                modelAndView.addObject("interviewAnalysis", interview.getStatus());
            }

            if(candidate.getFile() != null){
                modelAndView.addObject("file", candidate.getFile());
            }
            modelAndView.addObject("candidateId", candidateId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("uploadCandidateResume")
    public ModelAndView uploadCandidateResume(@RequestParam("file") MultipartFile file, @RequestParam("candidateId") int candidateId){
        log.info("candidateId: {}", candidateId);
        String BASE_URL = env.getProperty("baseUrl");

        ModelAndView modelAndView = new ModelAndView();
        try {
            Candidate candidate = candidateService.getCandidateById(candidateId);

            if (file.isEmpty()) {
                log.info("message: Please select a file to upload");
                modelAndView.addObject("file", candidate.getFile());
                modelAndView.setViewName("candidate/viewUpdateCandidateView");
                modelAndView.addObject("candidateId", candidateId);
                modelAndView.addObject("uploadCandidateResumeMessage", "Please select a file to upload!!!");

                return modelAndView;
            }

            /**
             * Restrict pdf documents
             * */
            String fileExtension = file.getOriginalFilename().split("\\.")[1];
            if(!fileExtension.equalsIgnoreCase("pdf")){
                log.info("message: You can only upload pdf files");
                modelAndView.setViewName("candidate/viewUpdateCandidateView");
                modelAndView.addObject("candidateId", candidateId);
                modelAndView.addObject("uploadCandidateResumeMessage", "You can only upload pdf files!!!");

                return modelAndView;
            }

            int leftLimit = 97; /** letter 'a' */
            int rightLimit = 122; /** letter 'z' */
            int targetStringLength = 6;
            Random random = new Random();
            StringBuilder buffer = new StringBuilder(targetStringLength);
            for (int i = 0; i < targetStringLength; i++) {
                int randomLimitedInt = leftLimit + (int)
                        (random.nextFloat() * (rightLimit - leftLimit + 1));
                buffer.append((char) randomLimitedInt);
            }
            String generatedString = buffer.toString();

            /** Get the file and save it somewhere */
            byte[] bytes = file.getBytes();
            String file_name = generatedString + "_" + file.getOriginalFilename();
            Path path = Paths.get(BASE_URL + UPLOADED_FOLDER + file_name);
            Files.write(path, bytes);

            log.info("File name: " + file_name);

            candidate.setFile(file_name);
            candidateService.updateCandidate(candidate);

            return viewAllCandidates(candidate.getJobId());
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            modelAndView.setViewName("candidate/viewUpdateCandidateView");
            modelAndView.addObject("candidateId", candidateId);
            modelAndView.addObject("uploadCandidateResumeMessage", "Error: " + e.getMessage());
        }

        return modelAndView;
    }

    @GetMapping("analyseResume/{candidateId}")
    public ModelAndView analyseResume(@PathVariable int candidateId){
        try{
            Candidate candidate = candidateService.getCandidateById(candidateId);
            candidate.setResumeAnalysis(CANDIDATE_STATUS_DO_ANALYSIS);

            candidateService.updateCandidate(candidate);

            return viewAllCandidates(candidate.getJobId());
        }
        catch (Exception e){
            ModelAndView modelAndView = new ModelAndView();

            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);

            return modelAndView;
        }
    }

    @GetMapping("viewUpdateCandidateImage/{candidateId}")
    public ModelAndView viewUpdateCandidateImage(@PathVariable int candidateId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            Candidate candidate = candidateService.getCandidateById(candidateId);

            modelAndView.setViewName("candidate/viewUpdateCandidateImage");
            modelAndView.addObject("text", APP_NAME);

            if(candidate.getFile() != null){
                modelAndView.addObject("image", candidate.getImage());
            }
            modelAndView.addObject("candidateId", candidateId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("uploadCandidateImage")
    public ModelAndView uploadCandidateImage(@RequestParam("image") MultipartFile image, @RequestParam("candidateId") int candidateId){
        log.info("candidateId: {}", candidateId);
        String BASE_URL = env.getProperty("baseUrl");

        ModelAndView modelAndView = new ModelAndView();
        try {
            Candidate candidate = candidateService.getCandidateById(candidateId);

            if (image.isEmpty()) {
                log.info("message: Please select an image to upload");
                modelAndView.addObject("image", candidate.getImage());
                modelAndView.setViewName("candidate/viewUpdateCandidateImage");
                modelAndView.addObject("candidateId", candidateId);
                modelAndView.addObject("uploadCandidateImageMessage", "Please select an image to upload!!!");

                return modelAndView;
            }

            /**
             * Restrict png or jpg or jpeg documents
             * */
            String fileExtension = image.getOriginalFilename().split("\\.")[1];
            if(!(fileExtension.equalsIgnoreCase("png")
                    || fileExtension.equalsIgnoreCase("jpg")
                    || fileExtension.equalsIgnoreCase("jpeg"))){
                log.info("message: You can only upload png or jpg images");
                modelAndView.setViewName("candidate/viewUpdateCandidateImage");
                modelAndView.addObject("candidateId", candidateId);
                modelAndView.addObject("uploadCandidateImageMessage", "You can only upload png or jpg images!!!");

                return modelAndView;
            }

            int leftLimit = 97; /** letter 'a' */
            int rightLimit = 122; /** letter 'z' */
            int targetStringLength = 6;
            Random random = new Random();
            StringBuilder buffer = new StringBuilder(targetStringLength);
            for (int i = 0; i < targetStringLength; i++) {
                int randomLimitedInt = leftLimit + (int)
                        (random.nextFloat() * (rightLimit - leftLimit + 1));
                buffer.append((char) randomLimitedInt);
            }
            String generatedString = buffer.toString();

            /** Get the image and save it somewhere */
            byte[] bytes = image.getBytes();
            String file_name = generatedString + "_" + image.getOriginalFilename();
            Path path = Paths.get(BASE_URL + UPLOADED_FOLDER_CANDIDATE_IMAGE + file_name);
            Files.write(path, bytes);

            log.info("File name: " + file_name);

            candidate.setImage(file_name);
            candidateService.updateCandidate(candidate);

            return viewAllCandidates(candidate.getJobId());
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            modelAndView.setViewName("candidate/viewUpdateCandidateImage");
            modelAndView.addObject("candidateId", candidateId);
            modelAndView.addObject("uploadCandidateImageMessage", "Error: " + e.getMessage());
        }

        return modelAndView;
    }

    @GetMapping("viewOrUpdateInterviewView/{candidateId}")
    public ModelAndView viewOrUpdateInterviewView(@PathVariable int candidateId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            Interview interview = interviewService.getInterviewByCandidateId(candidateId);
            Candidate candidate = candidateService.getCandidateById(candidateId);

            modelAndView.setViewName("candidate/viewOrUpdateInterviewView");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("interview", interview);
            modelAndView.addObject("candidate", candidate);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @GetMapping("updateInterviewSelectionStatus/{candidateId}/{status}")
    public ModelAndView updateInterviewSelectionStatus(@PathVariable int candidateId, @PathVariable String status){
        ModelAndView modelAndView = new ModelAndView();

        try{
            boolean update = interviewService.updateInterviewSelectionStatus(candidateId, status);
            if(update){
                modelAndView = viewOrUpdateInterviewView(candidateId);
            }
            else{
                throw new Exception("Invalid interview status: " + status);
            }
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }
}
