package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.CandidateService;
import lk.sliit.research.interviewapp.Service.InterviewService;
import lk.sliit.research.interviewapp.Service.QnaInterviewService;
import lk.sliit.research.interviewapp.model.domain.Candidate;
import lk.sliit.research.interviewapp.model.domain.Interview;
import lk.sliit.research.interviewapp.model.dto.request.GetQuestionRequestDto;
import lk.sliit.research.interviewapp.model.dto.response.GetQuestionResponseDto;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.ArrayList;
import java.util.List;

import static lk.sliit.research.interviewapp.util.Constant.*;

@Slf4j
@RestController
@RequestMapping("interview")
public class InterviewController {

    @Autowired
    InterviewService interviewService;

    @Autowired
    QnaInterviewService qnaInterviewService;

    @Autowired
    CandidateService candidateService;

    @GetMapping("enterKeyView")
    public ModelAndView enterKeyView(){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("interview/enterKeyView");
            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("enterInterview")
    public ModelAndView enterInterview(@RequestParam("roomId") String roomId, @RequestParam("key") String key){
        log.info("key: {}", key);
        ModelAndView modelAndView = new ModelAndView();

        try{
            Interview interview = interviewService.getInterviewByRoomIdAndKey(roomId, key);
            String question = "Hello candidate, welcome to the interview. I will be doing your interview. What is your name?";

            if(interview != null){
                interviewService.updateInterviewLifeCycle(interview.getId(), INTERVIEW_STATUS_PROCESSING);

                modelAndView.setViewName("interview/interviewPannel");
                modelAndView.addObject("question", question);
                modelAndView.addObject("interviewId", interview.getId());
            }
            else{
                Interview interviewNotInInitial = interviewService.findByRoomIdAndKeyNotInInitial(roomId, key);
                if(interviewNotInInitial == null){
                    modelAndView.setViewName("interview/enterKeyView");
                    modelAndView.addObject("error", "The key you entered is invalid, or the interview is not placed. Please contact ADMIN for more information");
                }
                else{
                    Candidate candidate = candidateService.getCandidateById(interviewNotInInitial.getCandidateId());
                    modelAndView.setViewName("interview/interviewResult");
                    modelAndView.addObject("interview", interviewNotInInitial);
                    modelAndView.addObject("candidate", candidate);
                }
            }

            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("getQuestion")
    public GetQuestionResponseDto getQuestion(@RequestBody GetQuestionRequestDto getQuestionRequestDto){
        GetQuestionResponseDto getQuestionResponseDto = new GetQuestionResponseDto();
        try{
            log.info("GetQuestionDto: {}", getQuestionRequestDto);

            qnaInterviewService.addInterviewAnswer(getQuestionRequestDto.getInterViewId(), getQuestionRequestDto.getPreviousQuestionId(), getQuestionRequestDto.getAnswer());

            if(isEndOfQuestion(getQuestionRequestDto)){
                interviewService.updateInterviewLifeCycle(getQuestionRequestDto.getInterViewId(), INTERVIEW_STATUS_INTERVIEW_OVER);

                getQuestionResponseDto.setQuestionId(-1);
                getQuestionResponseDto.setQuestion("end");

                return getQuestionResponseDto;
            }

            getQuestionResponseDto.setQuestionId(getQuestionRequestDto.getCount() + 1);
            getQuestionResponseDto.setQuestion(getMockQuestion(getQuestionRequestDto.getCount()));
            return getQuestionResponseDto;
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);

            getQuestionResponseDto.setQuestionId(0);
            getQuestionResponseDto.setQuestion(ERROR);
            return getQuestionResponseDto;
        }
    }

    /**
     * Function to decide end of question
     * Restriction for now: Restricted after question count is 10
     * */
    private boolean isEndOfQuestion(GetQuestionRequestDto getQuestionRequestDto){
        return getQuestionRequestDto.getCount() >= 10;
    }

    private String getMockQuestion(int id){
        List<String> mockQuestionList = new ArrayList<>();

        mockQuestionList.add("Tell me about your self");
        mockQuestionList.add("How old are you?");
        mockQuestionList.add("Where have you studied?");
        mockQuestionList.add("What is your hobby?");
        mockQuestionList.add("Who are in your family?");
        mockQuestionList.add("What is your father's name?");
        mockQuestionList.add("What is your mother's name?");
        mockQuestionList.add("What is your favorite thing to do?");
        mockQuestionList.add("What are you doing these days?");
        mockQuestionList.add("Do you like the questions asked?");

        return mockQuestionList.get(id);
    }

    @PostMapping("placeInterview")
    public String placeInterview(@RequestBody Interview interview){
        log.info("Interview: {}", interview);
        try{
            String eligibility = interviewService.checkInterviewEligible(interview.getCandidateId());
            if(!eligibility.equalsIgnoreCase(SUCCESS)){
                return eligibility;
            }

            interviewService.placeInterview(interview);
            return SUCCESS;
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return ERROR;
        }
    }

}
