package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.JobService;
import lk.sliit.research.interviewapp.model.domain.Job;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Random;

import static lk.sliit.research.interviewapp.util.Constant.*;

@Slf4j
@RestController
@RequestMapping("job")
public class JobController {

    private static String UPLOADED_FOLDER = "Interview-App-Git//interview-app//src//main//webapp//view//uploads//ads//";

    @Autowired
    JobService jobService;

    @Autowired
    Environment env;

    @GetMapping("addJobView")
    public ModelAndView addJobView(){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("job/addJobView");
            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @GetMapping("isUniqueName/{name}")
    public boolean isUniqueName(@PathVariable String name){
        log.info("Request: {}", name);

        try{
            return jobService.isUniqueName(name);
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return false;
        }
    }

    @PostMapping("createJob")
    public boolean createJob(@RequestBody Job job){
        log.info("Request: {}", job);

        try{
             return jobService.createJob(job);
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return false;
        }
    }

    @GetMapping("listAllJobsView")
    public ModelAndView listAllJobsView(){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.addObject("jobList", jobService.listAllJobs());
            modelAndView.setViewName("job/listAllJobs");
            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("listAllJobsFilter")
    public ModelAndView listAllJobsFilter(@RequestParam("name") String name){
        log.info("name: {}", name);

        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.addObject("jobList", jobService.filterJobsOnName(name));
            modelAndView.setViewName("job/listAllJobs");
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("uploadJobAds")
    public ModelAndView uploadJobAds(@RequestParam("file") MultipartFile file, @RequestParam("jobId") int jobId){
        log.info("jobId: {}", jobId);
        String BASE_URL = env.getProperty("baseUrl");

        ModelAndView modelAndView = new ModelAndView();
        try {
            Job job = jobService.getJobById(jobId);

            if (file.isEmpty()) {
                log.info("message: Please select a file to upload");
                modelAndView.setViewName("job/viewUpdateAdsView");
                modelAndView.addObject("jobId", jobId);
                modelAndView.addObject("file", job.getFile());
                modelAndView.addObject("uploadJobAdsMessage", "Please select a file to upload!!!");

                return modelAndView;
            }

            /**
             * Restrict pdf documents
             * */
            String fileExtension = file.getOriginalFilename().split("\\.")[1];
            if(!fileExtension.equalsIgnoreCase("pdf")){
                log.info("message: You can only upload pdf files");
                modelAndView.setViewName("job/viewUpdateAdsView");
                modelAndView.addObject("jobId", jobId);
                modelAndView.addObject("uploadJobAdsMessage", "You can only upload pdf files!!!");

                return modelAndView;
            }

            int leftLimit = 97; /** letter 'a' */
            int rightLimit = 122; /** letter 'z' */
            int targetStringLength = 6;
            Random random = new Random();
            StringBuilder buffer = new StringBuilder(targetStringLength);
            for (int i = 0; i < targetStringLength; i++) {
                int randomLimitedInt = leftLimit + (int)
                        (random.nextFloat() * (rightLimit - leftLimit + 1));
                buffer.append((char) randomLimitedInt);
            }
            String generatedString = buffer.toString();

            /** Get the file and save it somewhere */
            byte[] bytes = file.getBytes();
            String file_name = generatedString + "_" + file.getOriginalFilename();
            Path path = Paths.get(BASE_URL + UPLOADED_FOLDER + file_name);
            Files.write(path, bytes);

            log.info("File name: " + file_name);

            job.setFile(file_name);
            jobService.updateJob(job);

            return listAllJobsView();
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            modelAndView.setViewName("job/viewUpdateAdsView");
            modelAndView.addObject("jobId", jobId);
            modelAndView.addObject("uploadJobAdsMessage", "Error: " + e.getMessage());
        }

        return modelAndView;
    }

    @GetMapping("viewUpdateAdsView/{jobId}")
    public ModelAndView viewUpdateAdsView(@PathVariable int jobId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            Job job = jobService.getJobById(jobId);

            modelAndView.setViewName("job/viewUpdateAdsView");
            modelAndView.addObject("text", APP_NAME);

            if(job.getFile() != null){
                modelAndView.addObject("file", job.getFile());
            }
            modelAndView.addObject("jobId", jobId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }
}
